#ifndef GUARD_4976bbef3267846b59f3a26497ffe9db
#define GUARD_4976bbef3267846b59f3a26497ffe9db

#define USING_DUNE_PYTHON 1
#include <config.h>
#include <dune/alugrid/dgf.hh>
#include <dune/alugrid/grid.hh>
#include <dune/fem/gridpart/adaptiveleafgridpart.hh>
#include <dune/fem/gridpart/filter/simple.hh>
#include <dune/fem/gridpart/filteredgridpart.hh>
#include <dune/fempy/py/gridview.hh>
#include <dune/python/grid/gridview.hh>
#include <dune/python/grid/hierarchical.hh>
#include <dune/fem/function/localfunction/bindable.hh>
#include <dune/fem/common/intersectionside.hh>
#include <dune/python/pybind11/pybind11.h>
#include <dune/python/pybind11/extensions.h>
#include <dune/fempy/py/grid/gridpart.hh>
#include <dune/common/exceptions.hh>
#include <dune/fempy/function/virtualizedgridfunction.hh>
#include <dune/fempy/py/ufllocalfunction.hh>

namespace UFLLocalFunctions_4976bbef3267846b59f3a26497ffe9db
{

  // UFLLocalFunction
// ----------------

template< class GridPart, class Coeffbndproj >
struct UFLLocalFunction
  : public Dune::Fem::BindableGridFunctionWithSpace<GridPart,Dune::Dim<1>>
{
  typedef GridPart GridPartType;
  typedef typename GridPartType::GridViewType GridView;
  typedef typename GridView::ctype ctype;
  typedef Dune::Fem::BindableGridFunctionWithSpace<GridPart,Dune::Dim<1>> BaseType;
  typedef Dune::Fem::GridFunctionSpace<GridPartType,Dune::Dim<1>> FunctionSpaceType;
  typedef typename GridPartType::template Codim< 0 >::EntityType EntityType;
  typedef typename GridPartType::IntersectionType IntersectionType;
  typedef typename EntityType::Geometry Geometry;
  typedef typename Geometry::GlobalCoordinate GlobalCoordinateType;
  typedef Dune::Fem::IntersectionSide Side;
  typedef double Conepsilon;
  typedef std::tuple< std::shared_ptr< Conepsilon > > ConstantTupleType;
  template< std::size_t i >
  using ConstantsRangeType = typename std::tuple_element_t< i, ConstantTupleType >::element_type;
  typedef std::tuple< Dune::Fem::GridFunctionSpace< GridPartType, Dune::FieldVector< double, 2 > > > CoefficientFunctionSpaceTupleType;
  typedef std::tuple< Coeffbndproj > CoefficientTupleType;
  template< std::size_t i >
  using CoefficientFunctionSpaceType = std::tuple_element_t< i, CoefficientFunctionSpaceTupleType >;
  template< std::size_t i >
  using CoefficientRangeType = typename CoefficientFunctionSpaceType< i >::RangeType;
  template< std::size_t i >
  using CoefficientJacobianRangeType = typename CoefficientFunctionSpaceType< i >::JacobianRangeType;
  static constexpr bool gridPartValid = Dune::Fem::checkGridPartValid<GridPartType,Dune::Fem::ConstLocalFunction<Coeffbndproj>>();
  template< std::size_t i >
  using CoefficientType = std::tuple_element_t< i, CoefficientTupleType >;
  template< std::size_t i >
  using ConstantType = typename std::tuple_element_t< i, ConstantTupleType >::element_type;
  using BaseType::entity;
  using BaseType::geometry;

  UFLLocalFunction ( const GridPartType &gridPart, const std::string &name, int order, const Coeffbndproj &coeffbndproj, const Dune::Fem::ParameterReader &parameter = Dune::Fem::Parameter::container() )
    : BaseType(gridPart,name,order),
      coefficients_( Dune::Fem::ConstLocalFunction< Coeffbndproj >( coeffbndproj ) )
  {
    std::get< 0 >( constants_ ) = std::make_shared< Conepsilon >( (Conepsilon(0)) );
  }

  void bind ( const IntersectionType &intersection, Side side )
  {
    BaseType::bind(intersection, side);
    std::get< 0 >( coefficients_ ).bind( this->entity() );
  }

  void bind ( const EntityType &entity )
  {
    BaseType::bind(entity);
    std::get< 0 >( coefficients_ ).bind( this->entity() );
  }

  void unbind ()
  {
    BaseType::unbind();
    std::get< 0 >( coefficients_ ).unbind();
  }

  template< class Point >
  void evaluate ( const Point &x, typename FunctionSpaceType::RangeType &result ) const
  {
    using std::min;
    using std::sqrt;
    using std::tanh;
    double tmp0 = constant< 0 >();
    typename CoefficientFunctionSpaceType< 0 >::RangeType tmp1 = evaluateCoefficient< 0 >( x );
    const auto tmp2 = -0.4 + tmp1[ 0 ];
    const auto tmp3 = tmp1[ 1 ] * tmp1[ 1 ];
    const auto tmp4 = tmp1[ 0 ] * tmp1[ 0 ];
    const auto tmp5 = tmp4 + tmp3;
    const auto tmp6 = 1e-10 + tmp5;
    const auto tmp7 = std::sqrt( tmp6 );
    const auto tmp8 = -1.4 + tmp7;
    const auto tmp9 = std::min( tmp8, tmp2 );
    const auto tmp10 = 3 * tmp9;
    const auto tmp11 = tmp10 / tmp0;
    const auto tmp12 = std::tanh( tmp11 );
    const auto tmp13 = -1 * tmp12;
    const auto tmp14 = 1 + tmp13;
    const auto tmp15 = 0.5 * tmp14;
    const auto tmp16 = -1 * tmp15;
    const auto tmp17 = 1 + tmp16;
    const auto tmp18 = tmp17 * tmp15;
    result[ 0 ] = tmp18;
  }

  template< class Point >
  void jacobian ( const Point &x, typename FunctionSpaceType::JacobianRangeType &result ) const
  {
    using std::cosh;
    using std::min;
    using std::pow;
    using std::sqrt;
    using std::tanh;
    double tmp0 = constant< 0 >();
    typename CoefficientFunctionSpaceType< 0 >::RangeType tmp1 = evaluateCoefficient< 0 >( x );
    const auto tmp2 = -0.4 + tmp1[ 0 ];
    const auto tmp3 = tmp1[ 1 ] * tmp1[ 1 ];
    const auto tmp4 = tmp1[ 0 ] * tmp1[ 0 ];
    const auto tmp5 = tmp4 + tmp3;
    const auto tmp6 = 1e-10 + tmp5;
    const auto tmp7 = std::sqrt( tmp6 );
    const auto tmp8 = -1.4 + tmp7;
    const auto tmp9 = std::min( tmp8, tmp2 );
    const auto tmp10 = 3 * tmp9;
    const auto tmp11 = tmp10 / tmp0;
    const auto tmp12 = 2.0 * tmp11;
    const auto tmp13 = std::cosh( tmp12 );
    const auto tmp14 = 1.0 + tmp13;
    const auto tmp15 = std::cosh( tmp11 );
    const auto tmp16 = 2.0 * tmp15;
    const auto tmp17 = tmp16 / tmp14;
    const auto tmp18 = std::pow( tmp17, 2 );
    const auto tmp19 = 2 * tmp7;
    typename CoefficientFunctionSpaceType< 0 >::JacobianRangeType tmp20 = jacobianCoefficient< 0 >( x );
    const auto tmp21 = tmp1[ 1 ] * (tmp20[ 1 ])[ 0 ];
    const auto tmp22 = tmp21 + tmp21;
    const auto tmp23 = tmp1[ 0 ] * (tmp20[ 0 ])[ 0 ];
    const auto tmp24 = tmp23 + tmp23;
    const auto tmp25 = tmp24 + tmp22;
    const auto tmp26 = tmp25 / tmp19;
    const auto tmp27 = tmp26 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp28 = -1 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp29 = 1.0 + tmp28;
    const auto tmp30 = (tmp20[ 0 ])[ 0 ] * tmp29;
    const auto tmp31 = tmp30 + tmp27;
    const auto tmp32 = 3 * tmp31;
    const auto tmp33 = tmp32 / tmp0;
    const auto tmp34 = tmp33 * tmp18;
    const auto tmp35 = -1 * tmp34;
    const auto tmp36 = 0.5 * tmp35;
    const auto tmp37 = std::tanh( tmp11 );
    const auto tmp38 = -1 * tmp37;
    const auto tmp39 = 1 + tmp38;
    const auto tmp40 = 0.5 * tmp39;
    const auto tmp41 = -1 * tmp40;
    const auto tmp42 = 1 + tmp41;
    const auto tmp43 = tmp42 * tmp36;
    const auto tmp44 = -1 * tmp36;
    const auto tmp45 = tmp40 * tmp44;
    const auto tmp46 = tmp45 + tmp43;
    const auto tmp47 = tmp1[ 1 ] * (tmp20[ 1 ])[ 1 ];
    const auto tmp48 = tmp47 + tmp47;
    const auto tmp49 = tmp1[ 0 ] * (tmp20[ 0 ])[ 1 ];
    const auto tmp50 = tmp49 + tmp49;
    const auto tmp51 = tmp50 + tmp48;
    const auto tmp52 = tmp51 / tmp19;
    const auto tmp53 = tmp52 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp54 = (tmp20[ 0 ])[ 1 ] * tmp29;
    const auto tmp55 = tmp54 + tmp53;
    const auto tmp56 = 3 * tmp55;
    const auto tmp57 = tmp56 / tmp0;
    const auto tmp58 = tmp57 * tmp18;
    const auto tmp59 = -1 * tmp58;
    const auto tmp60 = 0.5 * tmp59;
    const auto tmp61 = tmp42 * tmp60;
    const auto tmp62 = -1 * tmp60;
    const auto tmp63 = tmp40 * tmp62;
    const auto tmp64 = tmp63 + tmp61;
    (result[ 0 ])[ 0 ] = tmp46;
    (result[ 0 ])[ 1 ] = tmp64;
  }

  template< class Point >
  void hessian ( const Point &x, typename FunctionSpaceType::HessianRangeType &result ) const
  {
    using std::cosh;
    using std::min;
    using std::pow;
    using std::sinh;
    using std::sqrt;
    using std::tanh;
    double tmp0 = constant< 0 >();
    typename CoefficientFunctionSpaceType< 0 >::RangeType tmp1 = evaluateCoefficient< 0 >( x );
    const auto tmp2 = -0.4 + tmp1[ 0 ];
    const auto tmp3 = tmp1[ 1 ] * tmp1[ 1 ];
    const auto tmp4 = tmp1[ 0 ] * tmp1[ 0 ];
    const auto tmp5 = tmp4 + tmp3;
    const auto tmp6 = 1e-10 + tmp5;
    const auto tmp7 = std::sqrt( tmp6 );
    const auto tmp8 = -1.4 + tmp7;
    const auto tmp9 = std::min( tmp8, tmp2 );
    const auto tmp10 = 3 * tmp9;
    const auto tmp11 = tmp10 / tmp0;
    const auto tmp12 = 2.0 * tmp11;
    const auto tmp13 = std::cosh( tmp12 );
    const auto tmp14 = 1.0 + tmp13;
    const auto tmp15 = std::cosh( tmp11 );
    const auto tmp16 = 2.0 * tmp15;
    const auto tmp17 = tmp16 / tmp14;
    const auto tmp18 = std::pow( tmp17, 2 );
    const auto tmp19 = 2 * tmp7;
    typename CoefficientFunctionSpaceType< 0 >::JacobianRangeType tmp20 = jacobianCoefficient< 0 >( x );
    const auto tmp21 = tmp1[ 1 ] * (tmp20[ 1 ])[ 0 ];
    const auto tmp22 = tmp21 + tmp21;
    const auto tmp23 = tmp1[ 0 ] * (tmp20[ 0 ])[ 0 ];
    const auto tmp24 = tmp23 + tmp23;
    const auto tmp25 = tmp24 + tmp22;
    const auto tmp26 = tmp25 / tmp19;
    const auto tmp27 = tmp26 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp28 = -1 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp29 = 1.0 + tmp28;
    const auto tmp30 = (tmp20[ 0 ])[ 0 ] * tmp29;
    const auto tmp31 = tmp30 + tmp27;
    const auto tmp32 = 3 * tmp31;
    const auto tmp33 = tmp32 / tmp0;
    const auto tmp34 = tmp33 * tmp18;
    const auto tmp35 = -1 * tmp34;
    const auto tmp36 = 0.5 * tmp35;
    const auto tmp37 = -1 * tmp36;
    const auto tmp38 = tmp37 * tmp36;
    const auto tmp39 = 2 * tmp26;
    const auto tmp40 = tmp39 * tmp26;
    const auto tmp41 = -1 * tmp40;
    typename CoefficientFunctionSpaceType< 0 >::HessianRangeType tmp42 = hessianCoefficient< 0 >( x );
    const auto tmp43 = tmp1[ 1 ] * ((tmp42[ 1 ])[ 0 ])[ 0 ];
    const auto tmp44 = (tmp20[ 1 ])[ 0 ] * (tmp20[ 1 ])[ 0 ];
    const auto tmp45 = tmp44 + tmp43;
    const auto tmp46 = tmp45 + tmp45;
    const auto tmp47 = tmp1[ 0 ] * ((tmp42[ 0 ])[ 0 ])[ 0 ];
    const auto tmp48 = (tmp20[ 0 ])[ 0 ] * (tmp20[ 0 ])[ 0 ];
    const auto tmp49 = tmp48 + tmp47;
    const auto tmp50 = tmp49 + tmp49;
    const auto tmp51 = tmp50 + tmp46;
    const auto tmp52 = tmp51 + tmp41;
    const auto tmp53 = tmp52 / tmp19;
    const auto tmp54 = tmp53 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp55 = ((tmp42[ 0 ])[ 0 ])[ 0 ] * tmp29;
    const auto tmp56 = tmp55 + tmp54;
    const auto tmp57 = 3 * tmp56;
    const auto tmp58 = tmp57 / tmp0;
    const auto tmp59 = tmp58 * tmp18;
    const auto tmp60 = std::sinh( tmp11 );
    const auto tmp61 = tmp33 * tmp60;
    const auto tmp62 = 2.0 * tmp61;
    const auto tmp63 = std::sinh( tmp12 );
    const auto tmp64 = 2.0 * tmp33;
    const auto tmp65 = tmp64 * tmp63;
    const auto tmp66 = tmp65 * tmp17;
    const auto tmp67 = -1 * tmp66;
    const auto tmp68 = tmp67 + tmp62;
    const auto tmp69 = tmp68 / tmp14;
    const auto tmp70 = 2 * tmp69;
    const auto tmp71 = tmp70 * tmp17;
    const auto tmp72 = tmp71 * tmp33;
    const auto tmp73 = tmp72 + tmp59;
    const auto tmp74 = -1 * tmp73;
    const auto tmp75 = 0.5 * tmp74;
    const auto tmp76 = -1 * tmp75;
    const auto tmp77 = std::tanh( tmp11 );
    const auto tmp78 = -1 * tmp77;
    const auto tmp79 = 1 + tmp78;
    const auto tmp80 = 0.5 * tmp79;
    const auto tmp81 = tmp80 * tmp76;
    const auto tmp82 = tmp81 + tmp38;
    const auto tmp83 = -1 * tmp80;
    const auto tmp84 = 1 + tmp83;
    const auto tmp85 = tmp84 * tmp75;
    const auto tmp86 = tmp85 + tmp38;
    const auto tmp87 = tmp86 + tmp82;
    const auto tmp88 = tmp1[ 1 ] * (tmp20[ 1 ])[ 1 ];
    const auto tmp89 = tmp88 + tmp88;
    const auto tmp90 = tmp1[ 0 ] * (tmp20[ 0 ])[ 1 ];
    const auto tmp91 = tmp90 + tmp90;
    const auto tmp92 = tmp91 + tmp89;
    const auto tmp93 = tmp92 / tmp19;
    const auto tmp94 = tmp93 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp95 = (tmp20[ 0 ])[ 1 ] * tmp29;
    const auto tmp96 = tmp95 + tmp94;
    const auto tmp97 = 3 * tmp96;
    const auto tmp98 = tmp97 / tmp0;
    const auto tmp99 = tmp98 * tmp18;
    const auto tmp100 = -1 * tmp99;
    const auto tmp101 = 0.5 * tmp100;
    const auto tmp102 = tmp37 * tmp101;
    const auto tmp103 = 2 * tmp93;
    const auto tmp104 = tmp103 * tmp26;
    const auto tmp105 = -1 * tmp104;
    const auto tmp106 = (tmp20[ 1 ])[ 0 ] * (tmp20[ 1 ])[ 1 ];
    const auto tmp107 = tmp1[ 1 ] * ((tmp42[ 1 ])[ 0 ])[ 1 ];
    const auto tmp108 = tmp107 + tmp106;
    const auto tmp109 = tmp108 + tmp108;
    const auto tmp110 = (tmp20[ 0 ])[ 0 ] * (tmp20[ 0 ])[ 1 ];
    const auto tmp111 = tmp1[ 0 ] * ((tmp42[ 0 ])[ 0 ])[ 1 ];
    const auto tmp112 = tmp111 + tmp110;
    const auto tmp113 = tmp112 + tmp112;
    const auto tmp114 = tmp113 + tmp109;
    const auto tmp115 = tmp114 + tmp105;
    const auto tmp116 = tmp115 / tmp19;
    const auto tmp117 = tmp116 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp118 = ((tmp42[ 0 ])[ 0 ])[ 1 ] * tmp29;
    const auto tmp119 = tmp118 + tmp117;
    const auto tmp120 = 3 * tmp119;
    const auto tmp121 = tmp120 / tmp0;
    const auto tmp122 = tmp121 * tmp18;
    const auto tmp123 = tmp98 * tmp60;
    const auto tmp124 = 2.0 * tmp123;
    const auto tmp125 = 2.0 * tmp98;
    const auto tmp126 = tmp125 * tmp63;
    const auto tmp127 = tmp126 * tmp17;
    const auto tmp128 = -1 * tmp127;
    const auto tmp129 = tmp128 + tmp124;
    const auto tmp130 = tmp129 / tmp14;
    const auto tmp131 = 2 * tmp130;
    const auto tmp132 = tmp131 * tmp17;
    const auto tmp133 = tmp132 * tmp33;
    const auto tmp134 = tmp133 + tmp122;
    const auto tmp135 = -1 * tmp134;
    const auto tmp136 = 0.5 * tmp135;
    const auto tmp137 = -1 * tmp136;
    const auto tmp138 = tmp80 * tmp137;
    const auto tmp139 = tmp138 + tmp102;
    const auto tmp140 = -1 * tmp101;
    const auto tmp141 = tmp140 * tmp36;
    const auto tmp142 = tmp84 * tmp136;
    const auto tmp143 = tmp142 + tmp141;
    const auto tmp144 = tmp143 + tmp139;
    const auto tmp145 = tmp39 * tmp93;
    const auto tmp146 = -1 * tmp145;
    const auto tmp147 = tmp1[ 1 ] * ((tmp42[ 1 ])[ 1 ])[ 0 ];
    const auto tmp148 = tmp106 + tmp147;
    const auto tmp149 = tmp148 + tmp148;
    const auto tmp150 = tmp1[ 0 ] * ((tmp42[ 0 ])[ 1 ])[ 0 ];
    const auto tmp151 = tmp110 + tmp150;
    const auto tmp152 = tmp151 + tmp151;
    const auto tmp153 = tmp152 + tmp149;
    const auto tmp154 = tmp153 + tmp146;
    const auto tmp155 = tmp154 / tmp19;
    const auto tmp156 = tmp155 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp157 = ((tmp42[ 0 ])[ 1 ])[ 0 ] * tmp29;
    const auto tmp158 = tmp157 + tmp156;
    const auto tmp159 = 3 * tmp158;
    const auto tmp160 = tmp159 / tmp0;
    const auto tmp161 = tmp160 * tmp18;
    const auto tmp162 = tmp71 * tmp98;
    const auto tmp163 = tmp162 + tmp161;
    const auto tmp164 = -1 * tmp163;
    const auto tmp165 = 0.5 * tmp164;
    const auto tmp166 = tmp84 * tmp165;
    const auto tmp167 = tmp166 + tmp102;
    const auto tmp168 = -1 * tmp165;
    const auto tmp169 = tmp80 * tmp168;
    const auto tmp170 = tmp169 + tmp141;
    const auto tmp171 = tmp170 + tmp167;
    const auto tmp172 = tmp140 * tmp101;
    const auto tmp173 = tmp103 * tmp93;
    const auto tmp174 = -1 * tmp173;
    const auto tmp175 = tmp1[ 1 ] * ((tmp42[ 1 ])[ 1 ])[ 1 ];
    const auto tmp176 = (tmp20[ 1 ])[ 1 ] * (tmp20[ 1 ])[ 1 ];
    const auto tmp177 = tmp176 + tmp175;
    const auto tmp178 = tmp177 + tmp177;
    const auto tmp179 = tmp1[ 0 ] * ((tmp42[ 0 ])[ 1 ])[ 1 ];
    const auto tmp180 = (tmp20[ 0 ])[ 1 ] * (tmp20[ 0 ])[ 1 ];
    const auto tmp181 = tmp180 + tmp179;
    const auto tmp182 = tmp181 + tmp181;
    const auto tmp183 = tmp182 + tmp178;
    const auto tmp184 = tmp183 + tmp174;
    const auto tmp185 = tmp184 / tmp19;
    const auto tmp186 = tmp185 * (tmp8 < tmp2 ? 1 : 0.0);
    const auto tmp187 = ((tmp42[ 0 ])[ 1 ])[ 1 ] * tmp29;
    const auto tmp188 = tmp187 + tmp186;
    const auto tmp189 = 3 * tmp188;
    const auto tmp190 = tmp189 / tmp0;
    const auto tmp191 = tmp190 * tmp18;
    const auto tmp192 = tmp132 * tmp98;
    const auto tmp193 = tmp192 + tmp191;
    const auto tmp194 = -1 * tmp193;
    const auto tmp195 = 0.5 * tmp194;
    const auto tmp196 = -1 * tmp195;
    const auto tmp197 = tmp80 * tmp196;
    const auto tmp198 = tmp197 + tmp172;
    const auto tmp199 = tmp84 * tmp195;
    const auto tmp200 = tmp199 + tmp172;
    const auto tmp201 = tmp200 + tmp198;
    ((result[ 0 ])[ 0 ])[ 0 ] = tmp87;
    ((result[ 0 ])[ 0 ])[ 1 ] = tmp144;
    ((result[ 0 ])[ 1 ])[ 0 ] = tmp171;
    ((result[ 0 ])[ 1 ])[ 1 ] = tmp201;
  }

  template< std::size_t i >
  const ConstantType< i > &constant () const
  {
    return *std::get< i >( constants_ );
  }

  template< std::size_t i >
  ConstantType< i > &constant ()
  {
    return *std::get< i >( constants_ );
  }

  const Conepsilon &conepsilon () const
  {
    return *std::get< 0 >( constants_ );
  }

  Conepsilon &conepsilon ()
  {
    return *std::get< 0 >( constants_ );
  }

  template< std::size_t i, class Point >
  typename CoefficientFunctionSpaceType< i >::RangeType evaluateCoefficient ( const Point &x ) const
  {
    typename CoefficientFunctionSpaceType< i >::RangeType result;
    std::get< i >( coefficients_ ).evaluate( x, result );;
    return result;
  }

  template< std::size_t i, class Point >
  typename CoefficientFunctionSpaceType< i >::JacobianRangeType jacobianCoefficient ( const Point &x ) const
  {
    typename CoefficientFunctionSpaceType< i >::JacobianRangeType result;
    std::get< i >( coefficients_ ).jacobian( x, result );;
    return result;
  }

  template< std::size_t i, class Point >
  typename CoefficientFunctionSpaceType< i >::HessianRangeType hessianCoefficient ( const Point &x ) const
  {
    typename CoefficientFunctionSpaceType< i >::HessianRangeType result;
    std::get< i >( coefficients_ ).hessian( x, result );;
    return result;
  }
  ConstantTupleType constants_;
  std::tuple< Dune::Fem::ConstLocalFunction< Coeffbndproj > > coefficients_;
};

} // namespace UFLLocalFunctions_4976bbef3267846b59f3a26497ffe9db

PYBIND11_MODULE( localfunction_4976bbef3267846b59f3a26497ffe9db_a524c1196983e65de1c06d7d6afdeb44, module )
{
  typedef UFLLocalFunctions_4976bbef3267846b59f3a26497ffe9db::UFLLocalFunction< typename Dune::FemPy::GridPart< Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true > >, Dune::FemPy::VirtualizedGridFunction< Dune::FemPy::GridPart<Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true >>, Dune::FieldVector< double, 2 > > > LocalFunctionType;
  if constexpr( LocalFunctionType::gridPartValid )
  {
      auto cls = Dune::Python::insertClass<LocalFunctionType>(module,"UFLLocalFunction",Dune::Python::GenerateTypeName("UFLLocalFunctions_4976bbef3267846b59f3a26497ffe9db::UFLLocalFunction< typename Dune::FemPy::GridPart< Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true > >, Dune::FemPy::VirtualizedGridFunction< Dune::FemPy::GridPart<Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true >>, Dune::FieldVector< double, 2 > > >"), Dune::Python::IncludeFiles({"python/dune/generated/localfunction_4976bbef3267846b59f3a26497ffe9db_a524c1196983e65de1c06d7d6afdeb44.cc"})).first;
      Dune::FemPy::registerUFLLocalFunction( module, cls );
      cls.def( pybind11::init( [] ( pybind11::object gridView, const std::string &name, int order, const Dune::FemPy::VirtualizedGridFunction< Dune::FemPy::GridPart<Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true >>, Dune::FieldVector< double, 2 > > &coeffbndproj ) {return new LocalFunctionType( Dune::FemPy::gridPart<Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true >>(gridView),name,order, coeffbndproj); } ), pybind11::keep_alive< 1, 2 >(), pybind11::keep_alive< 1, 3 >() );
      cls.def_property( "epsilon", [] ( LocalFunctionType &self ) -> UFLLocalFunctions_4976bbef3267846b59f3a26497ffe9db::UFLLocalFunction< typename Dune::FemPy::GridPart< Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true > >, Dune::FemPy::VirtualizedGridFunction< Dune::FemPy::GridPart<Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true >>, Dune::FieldVector< double, 2 > > >::Conepsilon { return self.conepsilon(); }, [] ( LocalFunctionType &self, const UFLLocalFunctions_4976bbef3267846b59f3a26497ffe9db::UFLLocalFunction< typename Dune::FemPy::GridPart< Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true > >, Dune::FemPy::VirtualizedGridFunction< Dune::FemPy::GridPart<Dune::Fem::FilteredGridPart< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > >, Dune::Fem::SimpleFilter< Dune::FemPy::GridPart< Dune::Fem::AdaptiveLeafGridPart< Dune::ALUGrid< 2, 2, Dune::simplex > > > >, true >>, Dune::FieldVector< double, 2 > > >::Conepsilon &v ) { self.conepsilon() = v; } );
      cls.def_property_readonly( "virtualized", [] ( LocalFunctionType& ) -> bool { return true;});
  }
}

#endif
